﻿using CaseAccountingContracts.BindingModels;
using CaseAccountingContracts.SearchModels;
using CaseAccountingContracts.StoragesContracts;
using CaseAccountingContracts.ViewModels;
using CaseAccountingDataBaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CaseAccountingDataBaseImplement.Implements
{
    public class LawyerStorage : ILawyerStorage
    {
        public LawyerViewModel? Delete(LawyerBindingModel model)
        {
            using var context = new CaseAccountingDatabase();
            var element = context.Lawyers
                .Include(x => x.Specialization)
                .Include(x => x.User)
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element == null)
            {
                return null;
            }
            context.Lawyers.Remove(element);
            context.SaveChanges();
            return element.GetViewModel;
        }

        public LawyerViewModel? GetElement(LawyerSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new CaseAccountingDatabase();
            return context.Lawyers
                .Include(x => x.Specialization)
                .Include(x => x.User)
                .FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)
                ?.GetViewModel;
        }

        public List<LawyerViewModel> GetFilteredList(LawyerSearchModel model)
        {
            using var context = new CaseAccountingDatabase();
            if (model.Id.HasValue)
            {
                return context.Lawyers
                    .Include(x => x.Specialization)
                    .Include(x => x.User)
                    .Where(x => x.Id == model.Id)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.UserId.HasValue)
            {
                return context.Lawyers
                    .Include(x => x.Specialization)
                    .Include(x => x.User)
                    .Where(x => x.UserId == model.UserId)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.SpecializationId.HasValue)
            {
                return context.Lawyers
                    .Include(x => x.Specialization)
                    .Include(x => x.User)
                    .Where(x => x.SpecializationId == model.SpecializationId)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else
            {
                return new();
            }
        }

        public List<LawyerViewModel> GetFullList()
        {
            using var context = new CaseAccountingDatabase();
            return context.Lawyers
                .Include(x => x.Specialization)
                .Include(x => x.User)
                .Select(x => x.GetViewModel).ToList();
        }

        public LawyerViewModel? Insert(LawyerBindingModel model)
        {
            var newLawyer = Lawyer.Create(model);
            if (newLawyer == null)
            {
                return null;
            }

            using var context = new CaseAccountingDatabase();

            context.Lawyers.Add(newLawyer);
            context.SaveChanges();
            return context.Lawyers
                         .Include(x => x.Specialization)
                         .Include(x => x.User)
                         .FirstOrDefault(x => x.Id == newLawyer.Id)
                         ?.GetViewModel;
        }

        public LawyerViewModel? Update(LawyerBindingModel model)
        {
            using var context = new CaseAccountingDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var lawyer = context.Lawyers
                .Include(x => x.Specialization)
                .Include(x => x.User)
                .FirstOrDefault(x => x.Id == model.Id);

                if (lawyer == null)
                {
                    return null;
                }
                lawyer.Update(model);
                context.SaveChanges();
                lawyer.UpdateContracts(context, model);
                return lawyer.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
    }
}
